#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <pthread.h>

#include "interface.h"
#include "support.h"
#include "utils.h"
#include "tpinterface.h"

PrintInfo* generate_print();

pthread_t   test;
int	    not_initiated = TRUE;

PrintInfo* generate_print()
{
    GtkWidget*	TCP_send_best = NULL;
    GtkWidget*	UDP_send_best = NULL;
    GtkWidget*	TCP_receive_best = NULL;
    GtkWidget*	UDP_receive_best = NULL;
    GtkWidget*	best_load_send = NULL;
    GtkWidget*	best_load_receive = NULL;
    PrintInfo*	print_info = NULL;

    if (my_state == MYSTATE_IDLE && engp)
    {
	print_info = g_malloc(sizeof(PrintInfo));
	g_return_if_fail(print_info != NULL);

	TCP_send_best = lookup_widget(mainwindow, "TCP_send_result");
        UDP_send_best = lookup_widget(mainwindow, "UDP_send_result");
	TCP_receive_best = lookup_widget(mainwindow, "TCP_receive_result");
	UDP_receive_best = lookup_widget(mainwindow, "UDP_receive_result");
	best_load_send = lookup_widget(mainwindow, "total_send");
        best_load_receive = lookup_widget(mainwindow, "total_receive");

	print_info->tcp_best_send = 
	    gtk_entry_get_text((GtkEntry *) TCP_send_best);
        print_info->tcp_best_receive = 
	    gtk_entry_get_text((GtkEntry *) TCP_receive_best);
	print_info->udp_best_send = 
	    gtk_entry_get_text((GtkEntry *) UDP_send_best);
        print_info->udp_best_receive = 
	    gtk_entry_get_text((GtkEntry *) UDP_receive_best);
	print_info->send_best_load = 
	    gtk_entry_get_text((GtkEntry *) best_load_send);
	print_info->receive_best_load = 
	    gtk_entry_get_text((GtkEntry *) best_load_receive);
	print_info->server = g_strdup_printf("%s:%d",
		engp->hostName, engp->hostCtrlPort);
	print_info->start_time = starttime;
	print_info->stop_time = stoptime;
    }

    return print_info;
}

void
on_print_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    PrintInfo*	print_info = NULL;

    print_info = generate_print();
    if (print_info)
	do_print(print_info, FALSE);
    g_free(print_info);
}


void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *dialog = NULL;
    
    dialog = create_aboutdialog();
    gtk_widget_show(dialog);
}


void
on_help_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    gchar* link = "http://www.internetkvalitetsguide.dk/StreamSpeed_Download.asp";

    activate_url(NULL, link, NULL);
}

int
delete_event			       (GtkMenuItem	*menuitem,
					gpointer	user_data )
{
   return FALSE;
}


void
on_quit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if (my_state != MYSTATE_IDLE)
    {
	pthread_cancel(test);
	StopContext(engp);
	update_testing(engp, FALSE);
	DeleteContext(engp);
    }
    gtk_main_quit();
}

void
on_preview_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    PrintInfo*	print_info = NULL;

    print_info = generate_print();
    if (print_info)
	do_print(print_info, TRUE);
    g_free(print_info);
}

void
on_button1_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    const gchar*    msg = NULL;
    GtkWidget*	    dialog = NULL;
    pthread_attr_t  attr;

    if (not_initiated)
    {
	if ((msg = initialize(FALSE)) == NULL)
	{
	    not_initiated = FALSE;
	    my_state = MYSTATE_IDLE;
	}
	else
	{
	    dialog = messagebox(
		_("Initialize"), msg,
		GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
	    gtk_dialog_run((GtkDialog *) dialog);
	    gtk_widget_destroy(dialog);
	    g_free(msg);
	    return;
	}
    }
    if (my_state == MYSTATE_IDLE)
    {
	my_state = MYSTATE_STDTEST;
	ClearBestRates(engp);
	clear_test_results(FALSE);
	gtk_button_set_label(button, _("_STOP"));
	pthread_attr_init(&attr);
	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
	if (pthread_create(&test, &attr, tptest, NULL))
	{
	    dialog = messagebox(
		_("Run test"), _("Unable to create new thread to run test"),
		GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
	    gtk_dialog_run((GtkDialog *) dialog);
	    gtk_widget_destroy(dialog);
	    my_state = MYSTATE_IDLE;
	}
    }
    else
    {
	my_state = MYSTATE_IDLE;
	ClearBestRates(engp);
	StopContext(engp);
	update_testing(engp, FALSE);
	clear_test_results(FALSE);
	gtk_button_set_label(button, _("_START"));
    }
}

