#include "utils.h"
#include "support.h"
#include "interface.h"

gchar* elapsed(double);

const gchar* const browser_list[][2] = {
    {"firefox", "-new-window"},
    {"konqueror", NULL},
    {"mozilla", "-new-window"},
    {"epiphany", "-new-window"},
    {NULL, NULL}
};

BrowserInfo my_browser;

gchar* elapsed(double times)
{
    gchar* elap = "0";
    short sec, min, hour;
    long ticks = (long) times;

    if (ticks > 0)
    {
	hour = (ticks % 3600)? ticks / 3600 : 0;
	if (hour)
	    ticks %= 3600;
	if (ticks % 3600)
	    min = (ticks % 60)? ticks / 60 : 0;
	else
	{
	    hour++;
	    min = 0;
	    ticks = 0;
	}
	if (min)
	{
	    ticks %= 60;
	    if (ticks % 60)
		sec = ticks;
	    else
	    {
		min++;
		sec = 0;
	    }
	}
	else
	{
	    sec = ticks % 60;
	    if (!sec && ticks)
		min++;
	}
	elap = g_strdup_printf("%02d:%02d:%02d", hour, min, sec);
    }
    return elap;
}

void do_print(PrintInfo* print_info, gboolean preview)
{
    FILE*	prt = NULL;
    int		result;
    gchar*	format = NULL;
    gchar*	teststart = NULL;
    gchar*	duration = NULL;
    DateInfo*	date = NULL;
    GtkWidget*	dialog = NULL;
    GtkWidget*	label = NULL;

    g_return_if_fail(print_info != NULL);
    date = get_date(SHORT, NULL);
    format = g_strdup_printf("\t\n%s - %s\n", date->date, date->time);
    duration = elapsed(difftime(print_info->stop_time, print_info->start_time));
    date = get_date(SHORT, &print_info->start_time);
    teststart = g_strdup_printf("%s %s", date->date, date->time);
    format = g_strconcat(format, _("\n\t\t\t\tTPTEST Session Report\n\n"), NULL);
    format = g_strdup_printf(
	    "%s\t\tTest Started:\t\t%s\n\t\tDuration:\t\t%s" 
	    "\n\t\tTest Server:\t\t%s\n\t\tBest UDP Send Rate:\t%s"
	    "\n\t\tBest UDP Receive Rate:\t%s"
	    "\n\t\tBest TCP Send Rate:\t%s"
	    "\n\t\tBest TCP Receive Rate:\t%s"
	    "\n\n\t\tSend Efficiency:\t%s"
	    "\n\t\tReceive Efficiency:\t%s\n",
	    format, teststart, duration, print_info->server,
	    print_info->udp_best_send, print_info->udp_best_receive,
	    print_info->tcp_best_send, print_info->tcp_best_receive,
	    print_info->send_best_load, print_info->receive_best_load);
    if (DEBUG)
	fprintf(stderr, "%s\n", format);
    if (preview)
    {
	g_return_if_fail(format != NULL);

	dialog = create_result_dialog();
	label = lookup_widget(dialog, "label18");
		
	gtk_label_set_text((GtkLabel *) label, format);
	int h,w;
	gtk_window_get_size((GtkWindow *) dialog, &w, &h);
	gtk_window_resize((GtkWindow *) dialog, 400, 200);
	gtk_window_get_size((GtkWindow *) dialog, &w, &h);
	gtk_window_set_modal((GtkWindow *) dialog, TRUE);
	gtk_widget_show(dialog);
	flush_display();
    }
    else
    {
	if ((prt = popen("lpr -T TPTEST -h -p", "w")) == NULL)
	    g_critical(_("Printing was not allowed"));
	fprintf(prt, format);
	fflush(prt);
	result = pclose(prt);
	if (!WIFEXITED(result))
	    g_warning(_("Abnormal exit"));
    }
    g_free(format);
    g_free(teststart);
    g_free(duration);
    g_free(date);
}

BrowserInfo get_browser()
{
    size_t i;
    DIR* dir = NULL;
    struct dirent* ent = NULL;
    gchar* path = NULL;
    gchar* file = NULL;

    my_browser.name = NULL;
    my_browser.option = NULL;
    for (i = 0; *browser_list[i] != NULL && !my_browser.name; i++)
    {
	if (DEBUG)
	    fprintf(stderr, "Looking for: %s\n", *browser_list[i]);
	if ((dir = opendir("/usr/bin")) == NULL)
	    if ((dir = opendir("/usr/local/bin")) == NULL)
		g_error(_("No access to user binaries"));
	    else
		path = "/usr/local/bin";
	else
	    path = "/usr/bin";

	while ((ent = readdir(dir)) != NULL)
	{
	    file = g_strdup(ent->d_name);
	    if ((strncmp(file, ".", PATH_MAX) == 0) ||
		    (strncmp(file, "..", PATH_MAX) == 0))
		continue;
	    if (strcmp(file, *browser_list[i]) == 0)
	    {
		my_browser.name = g_strdup_printf("%s/%s", path, file);
		my_browser.option = g_strdup(browser_list[i][1]);
		if (DEBUG)
		{
		    fprintf(stderr, "Browser found: %s\n", my_browser.name);
		    fprintf(stderr, "Using option: %s\n", my_browser.option);
		}
		break;
	    }
	}
	closedir(dir);
    }
    g_free(file);
    return my_browser;
}

void activate_url(GtkAboutDialog *dialog, const gchar *link, gpointer data)
{
    pid_t pid;

    g_return_if_fail(my_browser.name != NULL);
    g_return_if_fail(link != NULL);

    pid = fork();
    if (pid > 0)
    {
	execlp(my_browser.name, my_browser.name, my_browser.option, link , NULL);
	exit(0);
    }
}

DateInfo* get_date(DATETYPE type, time_t* myTime)
{
    struct tm* date = NULL;
    DateInfo* date_info = NULL;
    size_t n;
    time_t time_ticks;
    gchar *dateformat = NULL, *timeformat = NULL;
    gchar buf[BUFSIZ];

    dateformat = timeformat = NULL;
    switch (type) {
	case SHORT:
	    dateformat = g_strdup("%d/%m/%Y");
	    timeformat = g_strdup("%H:%M:%U");
	    break;
	case LONG:
	    dateformat = g_strdup("%A %d. %B %Y");
	    timeformat = g_strdup("%H:%M:%U");
	    break;
    }

    date_info = g_malloc(sizeof(DateInfo));
    time_ticks = time(myTime);
    date = localtime(&time_ticks);
    g_return_val_if_fail(date != NULL, date_info);
    
    n = strftime(buf, BUFSIZ, dateformat, date);
    date_info->date = g_strdup(buf);
    if (n == BUFSIZ)
    {
	date_info->date = NULL;
	g_error(_("Formatting time string exceeds maximum buffer size"));
    }
    n = strftime(buf, BUFSIZ, timeformat, date);
    date_info->time = g_strdup(buf);
    if (n == BUFSIZ)
    {
	date_info->time = NULL;
	g_error(_("Formatting time string exceeds maximum buffer size"));
    }
    g_free(timeformat);
    g_free(dateformat);
    return date_info;
}

GtkWidget* messagebox(
	const gchar* text, const gchar* subtext,
	GtkMessageType msg, GtkButtonsType btn)
{
    GtkWidget* dialog = NULL;

    dialog = gtk_message_dialog_new(
	    (GtkWindow *) mainwindow,
	    GTK_DIALOG_DESTROY_WITH_PARENT,
	    msg, btn, text);
    gtk_message_dialog_format_secondary_text(
	(GtkMessageDialog *) dialog, subtext);

    return dialog;
}

void init(void)
{
    /* For future improvements */
}
